#include "project.h"

enum {
    MAX_ITEMS = 10,
};

typedef enum  {
    TX_IDLE,
    TX_TRANSFERING,
} tx_state_tag;

volatile tx_state_tag tx_state = TX_IDLE;

typedef struct {
    uint8_t *const buffer;
    uint8_t cnt;
    uint8_t left_to_xfer;
} buffer_t;

uint8_t data_spi[MAX_ITEMS] = {0x01, 0x02, 0x03, 0x04, 0x05,
                              0x06, 0x07, 0x08, 0x09, 0x0A};

buffer_t tx_buffer = {
    .buffer = data_spi,
    .cnt = 0,
    .left_to_xfer = 0,
};

void spi_xfer_async(uint8_t *data, const size_t size);
void SPI_handler(void);

int main(void)
{
    isr_SPI_StartEx(SPI_handler);
    
    CyGlobalIntEnable;
    
    SPI_Start();
    UART_Start();
    
    UART_PutChar(0x0C); // clear the screen
    UART_PutString("Test SPI with interrupts.\r\n");

    while (1) {
        
        if (TX_IDLE == tx_state) {
            spi_xfer_async(tx_buffer.buffer, 3);
        }
        
        CyDelayUs(50);
        LED_Write(~LED_Read());
        
        if (TX_IDLE == tx_state) {
            spi_xfer_async(tx_buffer.buffer, 10);
        }
        
        CyDelayUs(100);
        LED_Write(~LED_Read());
    }
}

void spi_xfer_async(uint8_t *data, const size_t size)
{
    if (NULL != data) {
        CyGlobalIntDisable;
        if (TX_IDLE == tx_state) {
            tx_state = TX_TRANSFERING;
            
            if (size <= SPI_FIFO_SIZE) {
                // fill the spi tx fifo
                for (uint8_t i = 0; i < size; i++) {
                    SPI_WriteTxData(data[i]);
                }
            } else {
                tx_buffer.cnt = 2;
                tx_buffer.left_to_xfer = size;
                // put at least 2 bytes into the spi tx fifo
                for (uint8_t i = 0; i < 2; i++) {
                    SPI_WriteTxData(data[i]);
                }
            }
            
        }
        CyGlobalIntEnable;
    }
}

void SPI_handler(void)
{
    // clear interrupt flag
    volatile uint8_t sts = SPI_ReadTxStatus();
    
    // TODO: Get the data from MISO
    (void)SPI_ReadRxStatus();

    if (TX_IDLE != tx_state) {
        if (SPI_STS_BYTE_COMPLETE & sts) {
            // do we still have data to send?
            if (tx_buffer.left_to_xfer > tx_buffer.cnt) {
                SPI_WriteTxData(tx_buffer.buffer[tx_buffer.cnt]);
                tx_buffer.cnt++;
            }
        }
        // the interrupt was triggered by the SPI_DONE flag
        if (SPI_STS_SPI_IDLE & sts) {
            tx_state = TX_IDLE;
        }
    }
}

/* [] END OF FILE */
